import 'package:flutter/material.dart';
import 'package:sup/sup_defaults.dart';

import 'sup_stylist.dart';

export 'sup_config.dart';
export 'sup_stylist.dart';

enum SupAlignment { start, center, end }

class Sup extends StatelessWidget {
  final Widget image;
  final Widget title;
  final Widget subtitle;
  final Widget bottom;
  final SupAlignment alignment;
  final double spacing;

  final BoxDecoration boxDecoration;
  final EdgeInsets boxPadding;

  String _style;

  Sup({
    this.image,
    this.title,
    this.subtitle,
    this.bottom,
    this.alignment,
    this.boxDecoration,
    this.boxPadding,
    this.spacing,
  });

  factory Sup.styled(
    String style, {
    Widget image,
    Text title,
    Text subtitle,
    Widget bottom,
  }) {
    return Sup(
      image: image,
      title: title,
      subtitle: subtitle,
      bottom: bottom,
    ).._style = style;
  }

  _columnAlignmentFromSupAlignment(SupAlignment alignment) {
    switch (alignment) {
      case SupAlignment.start:
        return CrossAxisAlignment.start;
      case SupAlignment.center:
        return CrossAxisAlignment.center;
      case SupAlignment.end:
        return CrossAxisAlignment.end;
    }
  }

  @override
  Widget build(BuildContext context) {
    final stylist = SupStylist.of(context);
    final style = stylist.get(_style);
    final packageDefaults = SupDefaults.config;
    final userDefaults = stylist.defaults;
    // Override defaults with SupConfig from stylist
    final defaults = packageDefaults.merge(userDefaults ?? packageDefaults);
    // Override with matching style definition
    final styledDefaults = defaults.merge(style ?? defaults);
    // Override with passed params
    final values = styledDefaults.copyWith(
      image: image,
      title: title,
      subtitle: subtitle,
      bottom: this.bottom,
      alignment: alignment,
    );
    final bottom = values.bottom;

    final children = Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: _columnAlignmentFromSupAlignment(values.alignment),
      children: <Widget>[
        values.image,
        SizedBox(height: values.spacing),
        DefaultTextStyle(
          style: Theme.of(context).textTheme.title.copyWith(
                fontSize: 20,
                fontWeight: FontWeight.w700,
              ),
          textAlign: TextAlign.center,
          child: title ?? values.title,
        ),
        SizedBox(height: values.spacing),
        DefaultTextStyle(
          style: Theme.of(context).textTheme.caption.copyWith(
                fontSize: 18,
              ),
          textAlign: TextAlign.center,
          child: subtitle ?? values.subtitle,
        ),
        if (bottom != null) ...[
          SizedBox(height: values.spacing),
          bottom,
        ],
      ],
    );

    return children;
  }
}
