import 'package:flutter/widgets.dart';

import 'sup.dart';

class SupConfig {
  final Widget image;
  final Widget title;
  final Widget subtitle;
  final Widget bottom;
  final SupAlignment alignment;
  final String stackTraceDialogTitle;
  final String stackTraceOkAction;
  final String stackTraceCopyAction;
  final double spacing;

  const SupConfig({
    this.image,
    this.title,
    this.subtitle,
    this.bottom,
    this.alignment,
    this.stackTraceDialogTitle,
    this.stackTraceOkAction,
    this.stackTraceCopyAction,
    this.spacing,
  });

  SupConfig merge(SupConfig other) {
    if (other == null) return this;
    return copyWith(
      image: other.image,
      title: other.title,
      subtitle: other.subtitle,
      bottom: other.bottom,
      alignment: other.alignment,
      stackTraceDialogTitle: other.stackTraceDialogTitle,
      stackTraceOkAction: other.stackTraceOkAction,
      stackTraceCopyAction: other.stackTraceCopyAction,
      spacing: other.spacing,
    );
  }

  SupConfig copyWith({
    Widget image,
    Widget title,
    Widget subtitle,
    Widget bottom,
    SupAlignment alignment,
    TextStyle titleStyle,
    TextAlign titleAlign,
    TextStyle subtitleStyle,
    TextAlign subtitleAlign,
    String stackTraceDialogTitle,
    String stackTraceOkAction,
    String stackTraceCopyAction,
    double spacing,
  }) {
    return SupConfig(
      image: image ?? this.image,
      title: title ?? this.title,
      subtitle: subtitle ?? this.subtitle,
      bottom: bottom ?? this.bottom,
      alignment: alignment ?? this.alignment,
      stackTraceDialogTitle:
          stackTraceDialogTitle ?? this.stackTraceDialogTitle,
      stackTraceOkAction: stackTraceOkAction ?? this.stackTraceOkAction,
      stackTraceCopyAction: stackTraceCopyAction ?? this.stackTraceCopyAction,
      spacing: spacing ?? this.spacing,
    );
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is SupConfig &&
          runtimeType == other.runtimeType &&
          image == other.image &&
          title == other.title &&
          subtitle == other.subtitle &&
          bottom == other.bottom &&
          alignment == other.alignment &&
          stackTraceDialogTitle == other.stackTraceDialogTitle &&
          stackTraceOkAction == other.stackTraceOkAction &&
          stackTraceCopyAction == other.stackTraceCopyAction &&
          spacing == other.spacing;

  @override
  int get hashCode =>
      image.hashCode ^
      title.hashCode ^
      subtitle.hashCode ^
      bottom.hashCode ^
      alignment.hashCode ^
      stackTraceDialogTitle.hashCode ^
      stackTraceOkAction.hashCode ^
      stackTraceCopyAction.hashCode ^
      spacing.hashCode;
}
